//+------------------------------------------------------------------+
//|                                                Trend Filter3.mq5 |
//|                                  Copyright 2021, MetaQuotes Ltd. |
//|                                             https://www.mql5.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2021, MetaQuotes Ltd."
#property link      "https://www.mql5.com"
#property version   "1.00"
#include <MovingAverages.mqh>
#property indicator_separate_window
#property indicator_buffers 4
#property indicator_plots   1
#property indicator_type1   DRAW_COLOR_LINE
#property indicator_color1  clrYellow,clrLime,clrOrangeRed
#property indicator_style1  STYLE_SOLID
#property indicator_width1  2
#property indicator_level1  -0.9
#property indicator_level2   0
#property indicator_level3   0.9
#property indicator_minimum -1.05
#property indicator_maximum  1.05

input int  Nbars = 89;
input int  MA_Period = 9;

double iFish[], Value[], MA[], Color[];
//+------------------------------------------------------------------+
int OnInit()
  {
   SetIndexBuffer(0, iFish, INDICATOR_DATA);
   SetIndexBuffer(1, Color, INDICATOR_COLOR_INDEX);
   SetIndexBuffer(2, Value, INDICATOR_CALCULATIONS);
   SetIndexBuffer(3, MA, INDICATOR_CALCULATIONS);
   IndicatorSetString(INDICATOR_SHORTNAME, "TREND FILTER3 (" + (string)Nbars + "," + (string)MA_Period + ")");
   return(INIT_SUCCEEDED);
  }
//+------------------------------------------------------------------+
int OnCalculate(const int rates_total,
                const int prev_calculated,
                const datetime &time[],
                const double &open[],
                const double &high[],
                const double &low[],
                const double &close[],
                const long &tick_volume[],
                const long &volume[],
                const int &spread[])
  {
   int i, limit;
   double up, dn, osc;

   if(prev_calculated > 0)
      limit = prev_calculated;
   else
      limit = Nbars;

   for(i = limit; i < rates_total; i++)
     {
      up = high[iHighest(NULL, 0, MODE_HIGH, Nbars, rates_total - i)];
      dn = low[iLowest(NULL, 0, MODE_LOW, Nbars, rates_total - i)];

      if(up > dn)
         osc = 100 * (close[i] - dn) / (up - dn);
      else
         osc = 0;
      if(osc < 0)
         osc = 0.1;
      if(osc > 100)
         osc = 99.9;

      Value[i] = 0.1 * (osc - 50.0);
     }

   for(i = limit; i < rates_total; i++)
     {
      MA[i] = LinearWeightedMA(i, MA_Period, Value);

      iFish[i] = (MathExp(2.0 * MA[i]) - 1.0) / (MathExp(2.0 * MA[i]) + 1.0);

      if(iFish[i - 1] >= 0.9)
         Color[i] = 1;
      else
         if(iFish[i - 1] <= -0.9)
            Color[i] = 2;
     }
   return(rates_total);
  }
//+------------------------------------------------------------------+
